==> react-scrollwatcher-0.0.2.tgz: package/src/index.js <==
import React, {createContext, useCallback, useContext, useLayoutEffect, useReducer} from "react"
import SSR from "./ssr"

export const ScrollWatcherCTX = createContext()
export function useScrollWatch(){
    return useContext(ScrollWatcherCTX)
}
export function ScrollWatcherConsumer({children}){
    return (
      <ScrollWatcherCTX.Consumer>
        {children}
      </ScrollWatcherCTX.Consumer>
    )
}

function clientRect(node){
    const bcr = node.getBoundingClientRect()
    const clientT = document.documentElement.clientTop || document.body.clientTop || 0
    const clientL = document.documentElement.clientLeft || document.body.clientLeft || 0
    const {left:offsetX, top:offsetY} = getOrigin()

    return {
        left: node === document.documentElement || node === document.body ? offsetX : bcr.left + offsetX - clientL
      , top: node === document.documentElement || node === document.body ? offsetY : bcr.top + offsetY - clientT
      , width: bcr.width || bcr.right - bcr.left
      , contentWidth: node.scrollWidth
      , height: bcr.height || bcr.bottom - bcr.top
      , contentHeight: node.scrollHeight
    }
}

function compute(node, [referral, callback]){
    if ( document.documentElement.compareDocumentPosition(node) & Node.DOCUMENT_POSITION_DISCONNECTED )
      return

    if ( referral && ( referral.compareDocumentPosition(node) & Node.DOCUMENT_POSITION_DISCONNECTED ) )
      return

    if (!referral) {
        const { innerWidth:screenwidth, innerHeight:screenheight } = window
        const { left:originLeft, top:originTop } = getOrigin()
        const { width, contentWidth, left, height, contentHeight, top } = clientRect(node)
        const [offsetX, offsetY] = [ left-Math.abs(originLeft), top-Math.abs(originTop) ]
        const visibleWidth = ( offsetX > screenwidth ) || ( offsetX < -width )
          ? 0 : Math.min(width, screenwidth, screenwidth - offsetX, width + offsetX)
        const visibleHeight = ( offsetY > screenheight ) || ( offsetY < -height )
          ? 0 : Math.min(height, screenheight, screenheight - offsetY, height + offsetY)
        const coverageX = width === 0 ? 0 : visibleWidth/width
        const coverageY = height === 0 ? 0 : visibleHeight/height
        const coverage = (width*height) === 0 ? 0 : (visibleWidth*visibleHeight)/(width*height)
        const overflowX = -(screenwidth - offsetX - width)
        const overflowY = -(screenheight - offsetY - height)
        const progressX = 1 - (offsetX + width) / (width+screenwidth)
        const progressY = 1 - (offsetY + height) / (height+screenheight)
        const visibility = !!coverage

        callback({
            offsetX, offsetY
          , visibleWidth, visibleHeight
          , coverageX, coverageY, coverage
          , visibility
          , overflowX, overflowY
          , progressX, progressY
          , width, contentWidth
          , height, contentHeight
        })
    } else {
        const { innerWidth:screenwidth, innerHeight:screenheight } = window
        const { left:originLeft, top:originTop } = getOrigin()
        const { width:rwidth, left:rleft, height:rheight, top:rtop } = clientRect(referral)
        const { width, contentWidth, left, height, contentHeight, top } = clientRect(node)
        const roffsetX = rleft - Math.abs(originLeft)
        const roffsetY = rtop - Math.abs(originTop)
        const offsetX = left - Math.abs(originLeft)
        const offsetY = top - Math.abs(originTop)
        const rvisibleHeight = ( roffsetY > screenheight ) || ( roffsetY < -rheight )
          ? 0 : Math.min(rheight, screenheight, screenheight - roffsetY, rheight + roffsetY)
        const visibleHeight = !rvisibleHeight
          ? 0 : Math.max(0, Math.min(rtop+rvisibleHeight, top+height) - Math.max(rtop, top))
        const rvisibleWidth = ( roffsetX > screenwidth ) || ( roffsetX < -rwidth )
          ? 0 : Math.min(rwidth, screenwidth, screenwidth - roffsetX, rwidth + roffsetX)
        const visibleWidth = !rvisibleWidth
          ? 0 : Math.max(0, Math.min(rleft+rvisibleWidth, left+width) - Math.max(rleft, left))
        const coverageX = width === 0 ? 0 : visibleWidth/width
        const coverageY = height === 0 ? 0 : visibleHeight/height
        const coverage = (width*height) === 0 ? 0 : (visibleWidth*visibleHeight)/(width*height)
        const overflowX = -(screenwidth - offsetX - width)
        const overflowY = -(screenheight - offsetY - height)
        const progressX = 1 - (offsetX + width) / (width+screenwidth)
        const progressY = 1 - (offsetY + height) / (height+screenheight)
        const visibility = !!coverage

        callback({
            offsetX, offsetY
          , visibleWidth, visibleHeight
          , coverageX, coverageY, coverage
          , visibility
          , overflowX, overflowY
          , progressX, progressY
          , width, contentWidth
          , height, contentHeight
        })
    }
}

const domEventsHandler = e =>
  [...watched].forEach(([node, [referral, callback]]) =>
    compute(node, [referral, callback]))

export const directions = {
    STATIC: 0b0
  , UP: 0b1
  , DOWN: 0b10
  , LEFT: 0b100
  , RIGHT: 0B1000
}

function getOrigin(){
    return {
        left: window.pageXOffset || window.scrollX || document.documentElement.scrollLeft || document.body.scrollLeft || 0
      , top: window.pageYOffset || window.scrollY || document.documentElement.scrollTop || document.body.scrollTop || 0
    }
}

function matrixReducer(state, matrix){
    if (!matrix || !matrix.visibility)
      return undefined

    const {offsetX:prevOffsetX=0, offsetY:prevOffsetY=0} = state||{}
    const deltaX = Math.abs(matrix.offsetX) - Math.abs(prevOffsetX)
    const deltaY = Math.abs(matrix.offsetY) - Math.abs(prevOffsetY)
    const directionX = deltaX == 0
      ? directions.STATIC
        : deltaX > 0
          ? directions.LEFT
          : directions.RIGHT
    const directionY = deltaY == 0
      ? directions.STATIC
        : deltaY > 0
          ? directions.UP
          : directions.DOWN
    const direction = directionX ^ directionY

    if ( state && deltaX == 0 && deltaY == 0 && matrix.coverage == state.coverage )
      return state

    return {...matrix, deltaX, deltaY, direction}
}

let observerRaf
const observer = !SSR && window.MutationObserver
  ? new MutationObserver(() => {
        cancelAnimationFrame(observerRaf)
        observerRaf = requestAnimationFrame(() =>
          domEventsHandler({type: "dommutation"}))
    })
  : { observe: () => {}, disconnect: () => {} }

let watching = false
const watched = new Map()
function unwatch(node){
    watched.delete(node)

    if ( watching && !watched.size ) {
        window.removeEventListener("scroll", domEventsHandler)
        window.removeEventListener("resize", domEventsHandler)
        window.removeEventListener("orientationchange", domEventsHandler)
        window.removeEventListener("focus", domEventsHandler)
        observer.disconnect()
        watching = false
    }
}

function watch(node, [referral, callback]){
    watched.set(node, [referral, callback])

    if ( !watching && watched.size ) {
        window.addEventListener("scroll", domEventsHandler)
        window.addEventListener("resize", domEventsHandler)
        window.addEventListener("orientationchange", domEventsHandler)
        window.addEventListener("focus", domEventsHandler)
        observer.observe(document.documentElement, {childList: true, subtree: true })
        watching = true
    }

    if ( referral )
      compute(node, [referral, callback])
}

export default function ScrollWatcher({ children, watch:nodeRef, inside:referralRef, onOffsetChange}){
    const [matrix, setMatrix] = useReducer(matrixReducer, undefined)
    const callback = useCallback(matrix =>
      setMatrix(matrix)
    , [setMatrix])

    useLayoutEffect(() => {
        const node = !SSR && nodeRef && nodeRef.current && nodeRef.current.nodeType === Node.ELEMENT_NODE && nodeRef.current
        const referral = !SSR && referralRef && referralRef.current && referralRef.current.nodeType === Node.ELEMENT_NODE && referralRef.current

        node && watch(node, [referral, callback])
        return () =>
          node && unwatch(node, [referral, callback])
    }, [callback, matrix, nodeRef, referralRef, setMatrix])

    useLayoutEffect(() =>
      onOffsetChange && onOffsetChange(matrix)
    , [matrix, onOffsetChange])

    return (
      <ScrollWatcherCTX.Provider
          value={matrix}
      >
        {children}
      </ScrollWatcherCTX.Provider>
    )
}

==> react-scrollwatcher-0.0.5.tgz: package/src/index.js <==
import React, {createContext, useCallback, useContext, useLayoutEffect, useReducer} from "react"
import SSR from "./ssr"

export const ScrollWatcherCTX = createContext()
export function useScrollWatch(){
    return useContext(ScrollWatcherCTX)
}
export function ScrollWatcherConsumer({children}){
    return (
      <ScrollWatcherCTX.Consumer>
        {children}
      </ScrollWatcherCTX.Consumer>
    )
}

function clientRect(node){
    const bcr = node.getBoundingClientRect()
    const clientT = document.documentElement.clientTop || document.body.clientTop || 0
    const clientL = document.documentElement.clientLeft || document.body.clientLeft || 0
    const {left:offsetX, top:offsetY} = getOrigin()

    return {
        left: node === document.documentElement || node === document.body ? offsetX : bcr.left + offsetX - clientL
      , top: node === document.documentElement || node === document.body ? offsetY : bcr.top + offsetY - clientT
      , width: bcr.width || bcr.right - bcr.left
      , contentWidth: node.scrollWidth
      , height: bcr.height || bcr.bottom - bcr.top
      , contentHeight: node.scrollHeight
    }
}

function compute(node, [{dir, referral}, callback]){
    if ( document.documentElement.compareDocumentPosition(node) & Node.DOCUMENT_POSITION_DISCONNECTED )
      return

    if ( referral && ( referral.compareDocumentPosition(node) & Node.DOCUMENT_POSITION_DISCONNECTED ) )
      return

    if (!referral) {
        const { innerWidth:screenwidth, innerHeight:screenheight } = window
        const { left:originLeft, top:originTop } = getOrigin()
        const { width, contentWidth, left, height, contentHeight, top } = clientRect(node)
        const [offsetX, offsetY] = [ left-Math.abs(originLeft), top-Math.abs(originTop) ]
        const visibleWidth = ( offsetX > screenwidth ) || ( offsetX < -width )
          ? 0 : Math.min(width, screenwidth, screenwidth - offsetX, width + offsetX)
        const visibleHeight = ( offsetY > screenheight ) || ( offsetY < -height )
          ? 0 : Math.min(height, screenheight, screenheight - offsetY, height + offsetY)
        const coverageX = width === 0 ? 0 : visibleWidth/width
        const coverageY = height === 0 ? 0 : visibleHeight/height
        const coverage = (width*height) === 0 ? 0 : (visibleWidth*visibleHeight)/(width*height)
        const overflowX = -(screenwidth - offsetX - width)
        const overflowY = -(screenheight - offsetY - height)
        const progressX = 1 - ((dir=="ltr" ? offsetX : -overflowX) + width) / (width+screenwidth)
        const progressY = 1 - (offsetY + height) / (height+screenheight)
        const visibility = !!coverage

        callback({
            offsetX, offsetY
          , visibleWidth, visibleHeight
          , coverageX, coverageY, coverage
          , visibility
          , overflowX, overflowY
          , progressX, progressY
          , width, contentWidth
          , height, contentHeight
          , dir
        })
    } else {
        const { innerWidth:screenwidth, innerHeight:screenheight } = window
        const { left:originLeft, top:originTop } = getOrigin()
        const { width:rwidth, left:rleft, height:rheight, top:rtop } = clientRect(referral)
        const { width, contentWidth, left, height, contentHeight, top } = clientRect(node)
        const roffsetX = rleft - Math.abs(originLeft)
        const roffsetY = rtop - Math.abs(originTop)
        const offsetX = left - Math.abs(originLeft)
        const offsetY = top - Math.abs(originTop)
        const rvisibleHeight = ( roffsetY > screenheight ) || ( roffsetY < -rheight )
          ? 0 : Math.min(rheight, screenheight, screenheight - roffsetY, rheight + roffsetY)
        const visibleHeight = !rvisibleHeight
          ? 0 : Math.max(0, Math.min(rtop+rvisibleHeight, top+height) - Math.max(rtop, top))
        const rvisibleWidth = ( roffsetX > screenwidth ) || ( roffsetX < -rwidth )
          ? 0 : Math.min(rwidth, screenwidth, screenwidth - roffsetX, rwidth + roffsetX)
        const visibleWidth = !rvisibleWidth
          ? 0 : Math.max(0, Math.min(rleft+rvisibleWidth, left+width) - Math.max(rleft, left))
        const coverageX = width === 0 ? 0 : visibleWidth/width
        const coverageY = height === 0 ? 0 : visibleHeight/height
        const coverage = (width*height) === 0 ? 0 : (visibleWidth*visibleHeight)/(width*height)
        const overflowX = -(screenwidth - offsetX - width)
        const overflowY = -(screenheight - offsetY - height)
        const progressX = 1 - ((dir=="ltr" ? offsetX : -overflowX) + width) / (width+screenwidth)
        const progressY = 1 - (offsetY + height) / (height+screenheight)
        const visibility = !!coverage

        callback({
            offsetX, offsetY
          , visibleWidth, visibleHeight
          , coverageX, coverageY, coverage
          , visibility
          , overflowX, overflowY
          , progressX, progressY
          , width, contentWidth
          , height, contentHeight
          , dir
        })
    }
}

const domEventsHandler = e =>
  [...watched].forEach(([node, [referral, callback]]) =>
    compute(node, [referral, callback]))

export const directions = {
    STATIC: 0b0
  , UP: 0b1
  , DOWN: 0b10
  , LEFT: 0b100
  , RIGHT: 0B1000
}

function getOrigin(){
    return {
        left: window.pageXOffset || window.scrollX || document.documentElement.scrollLeft || document.body.scrollLeft || 0
      , top: window.pageYOffset || window.scrollY || document.documentElement.scrollTop || document.body.scrollTop || 0
    }
}

function matrixReducer(state, matrix){
    if (!matrix || !matrix.visibility)
      return undefined

    const {offsetX:prevOffsetX=0, offsetY:prevOffsetY=0} = state||{}
    const deltaX = Math.abs(matrix.offsetX) - Math.abs(prevOffsetX)
    const deltaY = Math.abs(matrix.offsetY) - Math.abs(prevOffsetY)
    const directionX = deltaX == 0
      ? directions.STATIC
        : deltaX > 0
          ? directions.LEFT
          : directions.RIGHT
    const directionY = deltaY == 0
      ? directions.STATIC
        : deltaY > 0
          ? directions.UP
          : directions.DOWN
    const direction = directionX ^ directionY

    if ( state && deltaX == 0 && deltaY == 0 && matrix.coverage == state.coverage )
      return state

    return {...matrix, deltaX, deltaY, direction}
}

let observerRaf
const observer = !SSR && window.MutationObserver
  ? new MutationObserver(() => {
        cancelAnimationFrame(observerRaf)
        observerRaf = requestAnimationFrame(() =>
          domEventsHandler({type: "dommutation"}))
    })
  : { observe: () => {}, disconnect: () => {} }

let watching = false
const watched = new Map()
function unwatch(node){
    watched.delete(node)

    if ( watching && !watched.size ) {
        window.removeEventListener("scroll", domEventsHandler)
        window.removeEventListener("resize", domEventsHandler)
        window.removeEventListener("orientationchange", domEventsHandler)
        window.removeEventListener("focus", domEventsHandler)
        observer.disconnect()
        watching = false
    }
}

function watch(node, [{dir, referral}, callback]){
    watched.set(node, [{dir, referral}, callback])

    if ( !watching && watched.size ) {
        window.addEventListener("scroll", domEventsHandler)
        window.addEventListener("resize", domEventsHandler)
        window.addEventListener("orientationchange", domEventsHandler)
        window.addEventListener("focus", domEventsHandler)
        observer.observe(document.documentElement, {childList: true, subtree: true })
        watching = true
    }

    if ( referral )
      compute(node, [{dir, referral}, callback])
}

export default function ScrollWatcher({ children, dir="ltr", watch:nodeRef, inside:referralRef, onOffsetChange}){
    const [matrix, setMatrix] = useReducer(matrixReducer, undefined)
    const callback = useCallback(matrix =>
      setMatrix(matrix)
    , [setMatrix])

    useLayoutEffect(() => {
        const node = !SSR && nodeRef && nodeRef.current && nodeRef.current.nodeType === Node.ELEMENT_NODE && nodeRef.current
        const referral = !SSR && referralRef && referralRef.current && referralRef.current.nodeType === Node.ELEMENT_NODE && referralRef.current

        node && watch(node, [{dir, referral}, callback])
        return () =>
          node && unwatch(node)
    }, [callback, dir, matrix, nodeRef, referralRef, setMatrix])

    useLayoutEffect(() =>
      onOffsetChange && onOffsetChange(matrix)
    , [matrix, onOffsetChange])

    return (
      <ScrollWatcherCTX.Provider
          value={matrix}
      >
        {children}
      </ScrollWatcherCTX.Provider>
    )
}
